@@ -28,6 +28,7 @@
 #' @param additional.covariates vector or data.frame, this parameter will be directly added to design
 #' matrix before running the differential analyses, therefore won't affect the batch corrections but
 #' adjust the results in down-stream analyses.
+#' @param sv.number number of surrogate variables to be calculated using SVA, best left untouched.
 #' @param run.enrichment logical, turns off enrichment pipeline
 #' @param enrichment.method There are two methodologies for enrichment analyses,
 #' Hyper-geometric p-value (HPEA) or Geneset Enrichment Analyses (GSEA).
@@ -38,7 +39,7 @@
 #' @param geneset Pathways to be used in enrichment analyses. If not set vp2008 (Chaussabel, 2008)
 #' immune modules will be used. This can be set to any geneset using `read.gmt` function from `qusage`
 #' package. Different modules are available: https://www.gsea-msigdb.org/gsea/downloads.jsp.
-#'
+#' @param verbose prints messages through running the pipeline
 #'
 #' @examples
 #' \donttest{
@@ -74,14 +75,19 @@ cinaR <-
            batch.correction = FALSE,
            batch.information = NULL,
            additional.covariates = NULL,
+           sv.number = NULL,
            run.enrichment = TRUE,
            enrichment.method = NULL,
            enrichment.FDR.cutoff = 1,
            background.genes.size = 20e3,
-           geneset = NULL) {
+           geneset = NULL,
+           verbose = TRUE) {
 
+    # Printing function
+    verbosePrint <- verboseFn(verbose)
+
+    verbosePrint(">> Experiment type: ", experiment.type)
 
-    message(">> Experiment type: ", experiment.type)
 
     if (is.null(reference.genome)) {
       warning("'reference.genome' is not set, therefore hg38 will be used!")
@@ -106,11 +112,13 @@ cinaR <-
       cp.filtered <-
         filterConsensus(cp, library.threshold = library.threshold, cpm.threshold = cpm.threshold)
 
+      verbosePrint(">> Matrix is filtered!")
 
       # annotate the peaks to the closest TSS
       cp.filtered.annotated <- annotatePeaks(cp.filtered,
                                              reference.genome = reference.genome,
-                                             show.annotation.pie = show.annotation.pie)
+                                             show.annotation.pie = show.annotation.pie,
+                                             verbose = verbose)
 
       # filter distance to TSS
       final.matrix <-
@@ -123,7 +131,7 @@ cinaR <-
         stop("Length of 'contrasts' must be equal to number of samples in `matrix`")
       }
 
-      message(">> Arranging count matrix...")
+      verbosePrint(">> Arranging count matrix...")
 
       # Remove spike-ins for now (may not exists in your data)
       matrix <- matrix [!grepl("^ERCC", matrix[,1]),]
@@ -151,6 +159,7 @@ cinaR <-
       final.matrix <-
         filterConsensus(matrix, library.threshold = library.threshold, cpm.threshold = cpm.threshold)
 
+      verbosePrint(">> Matrix is filtered!")
 
     } else {
       stop("`experiment.type` must be either 'ATAC-Seq' or 'RNA-Seq'")
@@ -159,7 +168,7 @@ cinaR <-
 
     if (!is.null(enrichment.method)) {
       if (enrichment.method == "GSEA" & run.enrichment == TRUE) {
-        message(
+        verbosePrint(
           ">> Setting `DA.fdr.threshold = 1` and `DA.lfc.threshold = 0`
               since GSEA is chosen for enrichment method!"
         )
@@ -179,9 +188,12 @@ cinaR <-
         DA.fdr.threshold = DA.fdr.threshold,
         DA.lfc.threshold = DA.lfc.threshold,
         save.DA.peaks = save.DA.peaks,
+        DA.peaks.path = DA.peaks.path,
         batch.correction = batch.correction,
         batch.information = batch.information,
-        additional.covariates = additional.covariates
+        additional.covariates = additional.covariates,
+        sv.number = sv.number,
+        verbose = verbose
       )
     } else {
       stop (
@@ -199,10 +211,11 @@ cinaR <-
           reference.genome = reference.genome,
           enrichment.method = enrichment.method,
           enrichment.FDR.cutoff = enrichment.FDR.cutoff,
-          background.genes.size = background.genes.size
+          background.genes.size = background.genes.size,
+          verbose = verbose
         )
-      message(">> Enrichment results are ready...")
-      message(">> Done!")
+      verbosePrint(">> Enrichment results are ready...")
+      verbosePrint(">> Done!")
       return(list(DA.results = DA.results,
                   Enrichment.Results = enrichment.results))
     }
@@ -242,7 +255,6 @@ filterConsensus <-
     } else {
       stop("filter.method should be either 'custom' or 'edgeR'")
     }
-    message(">> Matrix is filtered!")
     return(cp.filtered)
   }
 
@@ -292,18 +304,21 @@ normalizeConsensus <-
 #' @param cp bed formatted consensus peak matrix: CHR, START, STOP and raw peak counts (peaks by 3+samples)
 #' @param reference.genome genome of interested species. It should be 'hg38', 'hg19' or 'mm10'.
 #' @param show.annotation.pie shows the annotation pie chart produced with ChipSeeker
+#' @param verbose prints messages through running the pipeline
 #'
 #' @return DApeaks returns DA peaks
 annotatePeaks <-
   function(cp,
            reference.genome,
-           show.annotation.pie = FALSE) {
+           show.annotation.pie = FALSE,
+           verbose) {
+
+
     bed <-
       as.data.frame(do.call(rbind, strsplit(rownames(cp), "_", fixed = TRUE)))
     colnames(bed) <- c("CHR", "Start", "End")
     bed.GRanges <- GenomicRanges::GRanges(bed)
 
-
     if (reference.genome == "hg38") {
       if (!requireNamespace("TxDb.Hsapiens.UCSC.hg38.knownGene", quietly = TRUE)) {
         stop(
@@ -343,7 +358,8 @@ annotatePeaks <-
     }
 
     # annotate peaks
-    annoPeaks <- ChIPseeker::annotatePeak(bed.GRanges, TxDb = txdb, assignGenomicAnnotation = TRUE)
+    annoPeaks <- ChIPseeker::annotatePeak(bed.GRanges, TxDb = txdb, verbose = verbose)
+
 
     if (show.annotation.pie) {
       ChIPseeker::plotAnnoPie(annoPeaks)
@@ -352,6 +368,8 @@ annotatePeaks <-
     annoPeaks.anno <- annoPeaks@anno
     entrezids <- unique(annoPeaks.anno$geneId)
 
+
+
     # entrez to gene name mapping
     entrez2gene <-
       base::subset(genome,
@@ -362,7 +380,7 @@ annotatePeaks <-
     m <- match(annoPeaks.anno$geneId, entrez2gene$entrez)
     annoPeaks.anno$gene_name <- entrez2gene$symbol[m]
 
-    return(cbind(annoPeaks.anno, cp))
+    return(cbind(data.frame(annoPeaks.anno), cp))
   }
 
 #' Differential Analyses
@@ -386,7 +404,8 @@ annotatePeaks <-
 #' @param additional.covariates vector or data.frame, this parameter will be directly added to design
 #' matrix before running the differential analyses, therefore won't affect the batch corrections but
 #' adjust the results in down-stream analyses.
-#'
+#' @param sv.number number of surrogate variables to be calculated using SVA, best left untouched.
+#' @param verbose prints messages through running the pipeline
 #' @return returns consensus peaks (batch corrected version if enabled) and DA peaks
 differentialAnalyses <- function(final.matrix,
                                  contrasts,
@@ -398,7 +417,12 @@ differentialAnalyses <- function(final.matrix,
                                  DA.peaks.path,
                                  batch.correction,
                                  batch.information,
-                                 additional.covariates) {
+                                 additional.covariates,
+                                 sv.number,
+                                 verbose) {
+
+  # Printing function
+  verbosePrint <- verboseFn(verbose)
 
   # silence CRAN build notes
   log2FoldChange <- padj <- NULL
@@ -421,15 +445,20 @@ differentialAnalyses <- function(final.matrix,
       ## First normalize the consensus peaks to avoid detecting the effects
       ## confounding from library size as Michael Love and Jeff Leek suggests
       ## in this thread:
-      message(">> Running SVA for batch correction...")
+      verbosePrint(">> Running SVA for batch correction...")
 
       cp.metaless.normalized <- normalizeConsensus(cp.metaless, log.option = TRUE)
       mod  <- stats::model.matrix(~ 0 + contrasts)
       mod0 <- cbind(rep(1, length(contrasts)))
 
       # calculate the batch effects
-      sva.res <-
-        sva::svaseq(cp.metaless.normalized, mod, mod0)
+      if (is.null(sv.number)){
+        sva.res <-
+          sva::svaseq(cp.metaless.normalized, mod, mod0)
+      } else {
+        sva.res <-
+          sva::svaseq(cp.metaless.normalized, mod, mod0, n.sv = sv.number)
+      }
 
       # batch effect additional matrix
       add.batch <- sva.res$sv
@@ -446,7 +475,7 @@ differentialAnalyses <- function(final.matrix,
 
     } else {
       # if there is batch information available
-      message(">> Adding batch information to design matrix...")
+      verbosePrint(">> Adding batch information to design matrix...")
 
       if (nrow(design) != length(batch.information)) {
         stop("Number of samples and `batch.information` should be same length!")
@@ -472,7 +501,7 @@ differentialAnalyses <- function(final.matrix,
     }
     design <- cbind(design, additional.covariates)
 
-    message(">> Additional covariates are added to design matrix...")
+    verbosePrint(">> Additional covariates are added to design matrix...")
   }
 
   # Add intercept term for multiple comparisons
@@ -492,10 +521,6 @@ differentialAnalyses <- function(final.matrix,
     names(sort(contrasts.order[x]))
   })
 
-  contrast.names <-
-    apply(combs, 2, function(x) {
-      paste(x, collapse = "_")
-    })
   cc <- apply(combs, 2,
               function(x) {
                 paste0(paste(x, collapse = "_"), "=", x[1], "-", x[2])
@@ -519,7 +544,7 @@ differentialAnalyses <- function(final.matrix,
   if (DA.choice == 1) {
     ## edgeR
 
-    message(
+    verbosePrint(
       ">> Method: edgeR\n\tFDR:",
       DA.fdr.threshold,
       "& abs(logFC)<",
@@ -532,11 +557,11 @@ differentialAnalyses <- function(final.matrix,
     y <- edgeR::calcNormFactors(y, method = "TMM")
 
     # Estimate dispersion for genes with Bayesian Shrinkage
-    message(">> Estimating dispersion...")
+    verbosePrint(">> Estimating dispersion...")
     y <- edgeR::estimateDisp(y, design)
 
     # Fit the model
-    message(">> Fitting GLM...")
+    verbosePrint(">> Fitting GLM...")
     fit.glm <- edgeR::glmQLFit(y, design)
 
     for (i in seq_len(ncol(ccc))) {
@@ -574,7 +599,7 @@ differentialAnalyses <- function(final.matrix,
     }
   } else if (DA.choice == 2) {
     ## limma-voom
-    message(
+    verbosePrint(
       ">> Method: limma-voom\n\tFDR:",
       DA.fdr.threshold,
       "& abs(logFC)<",
@@ -625,7 +650,7 @@ differentialAnalyses <- function(final.matrix,
     }
   } else if (DA.choice == 3) {
     ## limma-trend
-    message(
+    verbosePrint(
       ">> Method: limma-trend\n\tFDR:",
       DA.fdr.threshold,
       "& abs(logFC)<",
@@ -674,7 +699,7 @@ differentialAnalyses <- function(final.matrix,
     }
   } else if (DA.choice == 4) {
     ## DEseq2
-    message(
+    verbosePrint(
       ">> Method: DEseq2\n\tFDR:",
       DA.fdr.threshold,
       "& abs(logFC)<",
@@ -735,15 +760,18 @@ differentialAnalyses <- function(final.matrix,
     }
   }
 
-  message(">> DA peaks are found!")
+  verbosePrint(">> DA peaks are found!")
 
   if (save.DA.peaks) {
+    # Make sure every list
+    DA.peaks.dfs <- lapply(DA.peaks, data.frame)
+
     if (is.null(DA.peaks.path)) {
-      message(">> Saving DA peaks to current directory as DApeaks.xlsx...")
-      writexl::write_xlsx(x = DA.peaks, path = "./DApeaks.xlsx")
+      verbosePrint(">> Saving DA peaks to current directory as DApeaks.xlsx...")
+      writexl::write_xlsx(x = DA.peaks.dfs, path = "./DApeaks.xlsx")
     } else {
-      message(paste0(">> Saving DA peaks to ", DA.peaks.path, "..."))
-      writexl::write_xlsx(x = DA.peaks, path = DA.peaks.path)
+      verbosePrint(paste0(">> Saving DA peaks to ", DA.peaks.path, "..."))
+      writexl::write_xlsx(x = DA.peaks.dfs, path = DA.peaks.path)
     }
   }
 
